// FRAP analysis macro for educational purposes
// 
// you find a more advanced version online:
// http://imagej.net/Analyze_FRAP_movies_with_a_Jython_script
//
// Usage:
// * Run FIJI (download it from http://fiji.sc)
// * Open your image
// * Open this script
// * Click the run button
//
// Author: Robert Haase, Scientific Computing Facility, MPI CBG Dresden
//         rhaase@mpi-cbg.de
// 
////////////////////////////////////////////////////////////////

// empty the results table to prevent result mixing
run("Clear Results");

// correct for bleaching
run("Bleach Correction", "correction=[Simple Ratio] background=0");
// register the stack to correct for motion of the sample
run("StackReg", "transformation=Translation");

// ask the user to select a region of interest
setTool("oval");
waitForUser("Please draw a circle in the frapped region.");

// read out size of the image stack (in time)
getDimensions(width, height, channels, slices, frames);

// configure measurement: we are only interested in mean 
// average signal intensity
run("Set Measurements...", "mean redirect=None decimal=2");

// measure the signal of the ROI in all frames. Put results 
// in a table
for (i = 1; i <= frames; i++) {
	Stack.setFrame(i);
	run("Measure");
} 

// read the table: determine mean signal intensity at the 
// beginning and the end of the image sequence
initialSignal = getResult("Mean", 0);
finalSignal = getResult("Mean", nResults() - 1);

// determine minimum signal intensity and the time point 
// when it was measured. This time point will be interpreted 
// as the time point when the laser bleached the sample.
minimumSignal = initialSignal;
minimumSignalTimePoint = i;
for (i = 1; i <= frames; i++) {
	signal = getResult("Mean", i-1);
	//print("i " + i + " " + signal);
	if (minimumSignal > signal) {
		minimumSignal = signal;
		minimumSignalTimePoint = i;
	}
}


// determine the time point (after bleaching) when the
// signal intensity reaches half of the level it has at 
// the end of the image sequence
halfRecoveredTimePoint = -1;
for (i = minimumSignalTimePoint; i <= frames; i++) {
	//print("i " + i);
	signal = getResult("Mean", i-1);
	if (signal > (minimumSignal + finalSignal / 2)) {
		halfRecoveredTimePoint = i;
		break;
	}
}

print("initialSignal " + initialSignal);
print("finalSignal " + finalSignal);
print("minimumSigna " + minimumSignal);

// from read out measurements, calculate the actual parameters
IF = (initialSignal - finalSignal) / (initialSignal - minimumSignal) * 100;
MF = (finalSignal - minimumSignal) / (initialSignal - minimumSignal) * 100;
CDT = (halfRecoveredTimePoint - minimumSignalTimePoint) * Stack.getFrameInterval();

Stack.getUnits(xUnit, yUnit, zUnit, timeUnit, valueUnit);

// show results
print("IF " + IF + "%");
print("MF " + MF + "%");
print("CDT " + CDT + timeUnit);

// plot the time course by filling two arrays (x and y values)
xValues = newArray();
yValues = newArray();
for (i = 1; i <= frames; i++) {
	signal = getResult("Mean", i-1);
	xValues = Array.concat(xValues, i);
	yValues = Array.concat(yValues, signal);
}

Plot.create("FRAP", "time", "signal", xValues, yValues);
Plot.show();
